package org.bayread.ckspecial;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Map;

import com.bayread.ckspecial.DBSpecial;
import com.bayread.ckspecial.core.annotation.Initial;
import com.bayread.ckspecial.db.ClickhouseDB;
import com.bayread.ckspecial.db.MysqlDB;
import com.bayread.ckspecial.db.annotation.DB;
import com.bayread.ckspecial.db.annotation.DBResource;
import com.bayread.ckspecial.db.annotation.SQL;
import com.bayread.ckspecial.db.annotation.SQLParam;
import org.bayread.ckspecial.bean.User;

/**
 * 以注解的形式操作数据库
 */
@DB("ck-read")
public abstract class AnnotationSamples {

	/**
	 * 如果数据库是MSYQL，可以用 "MysqlDB db"
	 */
	@DBResource("ck-read")
	ClickhouseDB db;

	@Initial
	public void init() throws IllegalAccessException {
		//先清理掉测试数据(正试环境慎用此方法)
		db.execute("truncate table users");

		List<User> users = new ArrayList<>();
		for(long i=10000;i<10010;i++){
			User user = new User();
			user.setUserid(i);
			user.setUsername("Martin".concat(i + ""));
			users.add(user);
		}
		db.writeBatch(users);

		System.out.println("");
		//--------------------------------------------------------------------------

		List<Integer> list = new ArrayList<>();
		list.add(10001);
		list.add(10002);
		list.add(10003);
		list.add(10004);

		List<Object[]> u = getObjectArrayList(list);
		u.forEach(e -> {
			for (Object to : e) {
				System.out.println(String.valueOf(to));
			}
		});

		System.out.println("");
		//--------------------------------------------------------------------------

		List<User> u1 = getBeanList(10003);
		u1.forEach(e -> e.print());

		System.out.println("");

		List<Map<String, Object>> u2 = getMapList(100);
		u2.forEach(e -> {
			e.forEach((k, v) -> {
				System.out.println(k + " " + v);
			});
		});

		System.out.println("");
		//--------------------------------------------------------------------------

		getBean(10000).print();

		System.out.println(Arrays.toString(getObjectArray(10000)));

		getMap(10000).forEach((k, v) -> {
			System.out.println(k + " " + v);
		});

		System.out.println("");
		//--------------------------------------------------------------------------

		System.out.println(getLong(10000));

		System.out.println("");
		//--------------------------------------------------------------------------

		System.out.println(set(2000, "djy"));

		System.out.println("");
		//--------------------------------------------------------------------------

		getUserById(2000).print();
	}

	@SQL(value = "select * from users where userid in ({userId})")
	public abstract List<Object[]> getObjectArrayList(@SQLParam("userId") List<Integer> k);

	@SQL(value = "select * from users where userid < {userId}")
	public abstract List<User> getBeanList(@SQLParam("userId") int k);

	@SQL(value = "select * from users where userid < {userId}")
	public abstract List<Map<String, Object>> getMapList(@SQLParam("userId") int k);

	@SQL(value = "select * from users where userid = {userId}")
	public abstract User getBean(@SQLParam("userId") int k);

	@SQL(value = "select * from users where userid = {userId}")
	public abstract Object[] getObjectArray(@SQLParam("userId") int k);

	@SQL(value = "select * from users where userid = {userId}")
	public abstract Map<String, Object> getMap(@SQLParam("userId") int k);

	@SQL(value = "select count(*) from users where userid > {userId}")
	public abstract Long getLong(@SQLParam("userId") int k);

	@SQL(value = "insert into users values({userId},{userName})")
	public abstract int set(@SQLParam("userId") int k, @SQLParam("userName") String v);

	public User getUserById(int uid) {
		return db.query("select * from users where userid = ?", User.class, uid);
	}

	public static void main(String[] args) {
		DBSpecial.run(AnnotationSamples.class);
	}
}