package org.bayread.ckspecial;

import com.bayread.ckspecial.commons.time.DateFormatUtils;
import com.bayread.ckspecial.db.ClickhouseDB;
import lombok.extern.slf4j.Slf4j;
import org.bayread.ckspecial.bean.LogItem;
import org.bayread.ckspecial.tool.IPUtil;

import java.io.IOException;
import java.sql.SQLException;
import java.util.*;

@Slf4j
public class Benchmark {
    public static List<LogItem> makeData(int count){
        List<LogItem> li = new ArrayList<>();
        Random r = new Random();
        for(int i=0;i<count;i++) {
            LogItem log = new LogItem();
            log.typeid=r.nextInt(10);
            log.resulttype =r.nextInt(20);;
            log.adnetwork=r.nextInt(5);;
            log.bidid=UUID.randomUUID().toString();
            log.timestamp=0l;
            log.ip= IPUtil.getRandomIp();
            log.os=r.nextInt(5);
            log.userid=UUID.randomUUID().toString().substring(0,8);
            log.preferdealid="";
            log.schedulingid="";
            log.dc=0;
            log.channelid=r.nextInt(50);
            log.mediaid="abc_" + log.channelid + "_" + r.nextInt(10);;
            log.placeid=log.mediaid + "_" + r.nextInt(100);
            log.genderid=0;
            log.ageid=0;
            log.citycode=r.nextInt(1000);
            log.advertiserid=r.nextInt(100);;
            log.campaignid=r.nextInt(10000);;
            log.groupid=r.nextInt(1000);;
            log.adid=r.nextInt(10000);;
            log.creativeid=r.nextInt(1000000);;
            log.price=r.nextInt(5000000);;
            log.agentprice=log.price;
            log.bidprice=log.price;
            log.day=DateFormatUtils.GENERIC_DATETIME_TODAY_FORMAT.format(new Date());
            log.hour=r.nextInt(23);
            log.minute=r.nextInt(59);
            li.add(log);
        }
        return li;
    }

    public static void main(String[] args) throws IOException, SQLException, InterruptedException {
        if(args.length != 3)
            args = new String[]{"3", "1000", "100"};
        int threadCount = Integer.parseInt(args[0]);
        int batchSize = Integer.parseInt(args[1]);
        int sleep = Integer.parseInt(args[2]);

        ClickhouseDB db= new ClickhouseDB("ck-sharding-write");
        for(int i=0;i<threadCount;i++) {
            new Thread(()->{
               while(true){
                   long beginTime = System.currentTimeMillis();
                   db.writeBatch(Benchmark.makeData(batchSize));
                   long useTime = System.currentTimeMillis() - beginTime;

                   log.info("时间:{} 批量大小[{}] 用时:[{}]",DateFormatUtils.GENERIC_DATETIME_FORMAT.format(new Date()),Thread.currentThread().getName(),batchSize,useTime);

                   try {
                       Thread.sleep(sleep);
                   } catch (InterruptedException e) {
                       throw new RuntimeException(e);
                   }
               }
            }).start();
        }
    }
}
